/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
//---------------------------
// PSoC5 Includes
//---------------------------
#include <device.h>
//---------------------------
// RTOS Includes
//---------------------------
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
//----------------------------------
// Define FreeRTOS related Resource
//----------------------------------
#define TASK_STACK_LED   128
#define TASK_PRIORITY_LED (tskIDLE_PRIORITY+1)
//
void Task_LED   (void *pvParameters);
//---------------------------
// Define Prototype
//---------------------------
void prvHardwareSetup(void);

//=======================================
// Hooks for Illegal Operations
//=======================================
void vApplicationStackOverflowHook(
         xTaskHandle pxTask,
         signed char *pcTaskName)
{
    // The stack space has been exceeded
    // for a task, so, allocate more.
    taskDISABLE_INTERRUPTS();
	for( ;; );
}
void vApplicationMallocFailedHook(void)
{
    // The heap space has been exceeded.
    taskDISABLE_INTERRUPTS();
	for( ;; );
}

//======================================
// Hardware Setup
//======================================
void prvHardwareSetup(void)
{
    // Port layer functions that need
    // to be copied into the vector table.
    extern void xPortPendSVHandler(void);
    extern void xPortSysTickHandler(void);
    extern void vPortSVCHandler(void);
    extern cyisraddress CyRamVectors[];

    // Install the OS Interrupt Handlers.
    CyRamVectors[11]
      = (cyisraddress) vPortSVCHandler;
    CyRamVectors[14]
      = (cyisraddress) xPortPendSVHandler;
    CyRamVectors[15]
      = (cyisraddress) xPortSysTickHandler;

    // Initialize Your Hardware
    //     later do it.
}

//=====================================
// Main Routine
//=====================================
void main()
{
    //--------------------------------------
    // Initialize Hardware related to RTOS
    //--------------------------------------
    prvHardwareSetup();
    CYGlobalIntEnable; // Enable Interrupts
    //-------------------
    // Initialize API
    //-------------------
    // none
    //-------------------------------------
    // Create Task
    //-------------------------------------
    xTaskCreate(Task_LED,                 \
                (signed portCHAR *)"LED", \
                TASK_STACK_LED,           \
                NULL,                     \
                TASK_PRIORITY_LED,        \
                NULL);
    //-------------------
    // Start RTOS Kernel 
    //-------------------
    vTaskStartScheduler();
    while(1);
}

//===========================
// Task : LED
//===========================
void Task_LED(void *pvParameters)
{
    // Block for 500ms
    const portTickType xDelay
            = 500 / portTICK_RATE_MS;
    // Task Loop
    while(1)
    {
        // Toggle LED
        LED_Write(~LED_Read());
        // Block  me
        vTaskDelay(xDelay);
    }
}


/* [] END OF FILE */
